"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TTLMap_1 = require("./TTLMap");
class MemoryLayer {
    constructor(options = {}) {
        this._map = new TTLMap_1.default();
        this._options = Object.assign({ ttl: Infinity, keyFn(key) {
                return key;
            } }, options);
    }
    get map() {
        return this._map;
    }
    async batchGet(keys) {
        return keys.map(key => {
            const value = this._map.get(this._options.keyFn(key));
            if (!value) {
                return new Error(`No cache value of key: ${key}`);
            }
            return value;
        });
    }
    async batchSet(map) {
        map.forEach((value, key) => {
            this._map.setTTL(this._options.keyFn(key), value, this._options.ttl);
        });
    }
}
exports.default = MemoryLayer;
//# sourceMappingURL=MemoryLayer.js.map